@@ -55,6 +55,12 @@ sub run {
         );
     }
 
+    if (not $self->{config}->{'scan-profiles'}) {
+        $self->{logger}->debug(
+            "--scan-profiles missing. Don't scan user profiles"
+        );
+    }
+
     if (not $ENV{PATH}) {
         # set a minimal PATH if none is set (#1129, #1747)
         $ENV{PATH} =
@@ -180,6 +186,7 @@ sub _initModulesList {
                 logger        => $self->{logger},
                 registry      => $self->{registry},
                 scan_homedirs => $self->{config}->{'scan-homedirs'},
+                scan_profiles => $self->{config}->{'scan-profiles'},
             }
         );
         if (!$enabled) {
@@ -262,6 +269,7 @@ sub _runModule {
             logger        => $self->{logger},
             registry      => $self->{registry},
             scan_homedirs => $self->{config}->{'scan-homedirs'},
+            scan_profiles => $self->{config}->{'scan-profiles'},
         }
     );
     $self->{modules}->{$module}->{done} = 1;
