@@ -20,11 +20,11 @@ sub doInventory {
     my $logger    = $params{logger};
 
     my @zones =
-        grep { !/global/ }
         getAllLines(command => '/usr/sbin/zoneadm list -p', logger => $logger);
 
     foreach my $zone (@zones) {
         my ($zoneid, $zonename, $zonestatus, undef, $uuid) = split(/:/, $zone);
+        next if $zonename eq 'global';
 
         # Memory considerations depends on rcapd or project definitions
         # Little hack, I go directly in /etc/zones reading mcap physcap for each zone.
